@@ -23,25 +23,27 @@
 #include <algorithm>  // for sort
 #include <string>     // for std::string
 
-#include "libsemigroups/froidure-pin-base.hpp"     // for FroidurePinBase
-#include "libsemigroups/libsemigroups-config.hpp"  // for LIBSEMIGROUPS_DEBUG
-#include "libsemigroups/libsemigroups-exception.hpp"  // for LIBSEMIGROUPS_EXCEPTION
-#include "libsemigroups/report.hpp"                   // for REPORT_DEFAULT ...
-#include "libsemigroups/string.hpp"                   // for detail::to_string
+#include "libsemigroups/config.hpp"             // for LIBSEMIGROUPS_DEBUG
+#include "libsemigroups/exception.hpp"          // for LIBSEMIGROUPS_EXCEPTION
+#include "libsemigroups/froidure-pin-base.hpp"  // for FroidurePinBase
+#include "libsemigroups/report.hpp"             // for REPORT_DEFAULT ...
+#include "libsemigroups/string.hpp"             // for detail::to_string
 
 namespace libsemigroups {
 
-  char to_human_readable(char c) noexcept {
-    if (c > 95) {
-      return c;
-    } else {
+  namespace {
+    char to_human_readable(char c) noexcept {
+      if (c > 95) {
+        return c;
+      } else {
 #ifndef LIBSEMIGROUPS_DEBUG
-      return c + 96;
+        return c + 96;
 #else
-      return c;
+        return c;
 #endif
+      }
     }
-  }
+  }  // namespace
 
   //////////////////////////////////////////////////////////////////////////////
   // FpSemigroupInterface - constructor + destructor - public
@@ -61,6 +63,8 @@ namespace libsemigroups {
         _is_obviously_finite(false),
         _is_obviously_infinite(false) {}
 
+  FpSemigroupInterface::~FpSemigroupInterface() = default;
+
   ////////////////////////////////////////////////////////////////////////////
   // Runner - pure virtual overridden function - public
   ////////////////////////////////////////////////////////////////////////////
@@ -110,15 +114,15 @@ namespace libsemigroups {
     reset();
   }
 
-  void FpSemigroupInterface::set_alphabet(size_t const nr_letters) {
+  void FpSemigroupInterface::set_alphabet(size_t number_of_letters) {
     if (!_alphabet.empty()) {
       LIBSEMIGROUPS_EXCEPTION("the alphabet cannot be set more than once");
-    } else if (nr_letters == 0) {
+    } else if (number_of_letters == 0) {
       LIBSEMIGROUPS_EXCEPTION("the alphabet must be non-empty");
-    } else if (nr_letters > 256) {
+    } else if (number_of_letters > 256) {
       LIBSEMIGROUPS_EXCEPTION("the alphabet must contain at most 256 letters");
     }
-    for (size_t i = 0; i < nr_letters; ++i) {
+    for (size_t i = 0; i < number_of_letters; ++i) {
 #ifdef LIBSEMIGROUPS_DEBUG
       _alphabet += static_cast<char>(i + 97);
 #else
@@ -126,14 +130,14 @@ namespace libsemigroups {
 #endif
       _alphabet_map.emplace(_alphabet[i], i);
     }
-    set_alphabet_impl(nr_letters);
+    set_alphabet_impl(number_of_letters);
     reset();
   }
 
   void FpSemigroupInterface::add_rules(FroidurePinBase& S) {
-    if (!_alphabet.empty() && _alphabet.size() != S.nr_generators()) {
+    if (!_alphabet.empty() && _alphabet.size() != S.number_of_generators()) {
       LIBSEMIGROUPS_EXCEPTION("incompatible number of generators, found "
-                              + detail::to_string(S.nr_generators())
+                              + detail::to_string(S.number_of_generators())
                               + ", should be at most "
                               + detail::to_string(_alphabet.size()));
     }
@@ -181,6 +185,7 @@ namespace libsemigroups {
                               + " but found " + detail::to_string(inv.size()));
     }
 
+    // Checks the inv consists of valid letters in the alphabet
     validate_word(inv);
 
     std::string cpy = inv;
@@ -193,11 +198,36 @@ namespace libsemigroups {
       }
     }
 
+    // Check that (x ^ - 1) ^ -1 = x
+    for (size_t i = 0; i < _alphabet.size(); ++i) {
+      if (_alphabet[i] == identity()[0] && inv[i] != identity()[0]) {
+        LIBSEMIGROUPS_EXCEPTION(
+            "invalid inverses, the identity is %c, but %c ^ -1 = %c",
+            _alphabet[i],
+            _alphabet[i],
+            inv[i]);
+      }
+      for (size_t j = 0; j < _alphabet.size(); ++j) {
+        if (_alphabet[j] == inv[i]) {
+          if (inv[j] != _alphabet[i]) {
+            LIBSEMIGROUPS_EXCEPTION(
+                "invalid inverses, %c ^ -1 = %c but %c ^ -1 = %c",
+                _alphabet[i],
+                inv[i],
+                inv[i],
+                inv[j]);
+          }
+          break;
+        }
+      }
+    }
+
     _inverses = inv;
 
     for (size_t i = 0; i < _alphabet.size(); ++i) {
-      add_rule(std::string(1, _alphabet[i]) + _inverses[i], _identity);
-      add_rule(std::string(1, _inverses[i]) + _alphabet[i], _identity);
+      if (_alphabet[i] != identity()[0]) {
+        add_rule(std::string(1, _alphabet[i]) + _inverses[i], _identity);
+      }
     }
   }
 
@@ -235,7 +265,7 @@ namespace libsemigroups {
     // general, so we leave the answer to this question to
     // is_quotient_obviously_infinite_impl in the derived class.
     if (alphabet().empty()) {
-      // If nr_generators() is undefined, then there is no quotient yet,
+      // If number_of_generators() is undefined, then there is no quotient yet,
       // and so it is not obviously infinite, or anything!
       REPORT_VERBOSE_DEFAULT("not obviously infinite (no alphabet defined)\n");
       return false;
@@ -320,7 +350,7 @@ namespace libsemigroups {
       LIBSEMIGROUPS_EXCEPTION("no alphabet has been defined");
     } else if (_alphabet_map.find(c) == _alphabet_map.end()) {
       LIBSEMIGROUPS_EXCEPTION(
-          "invalid letter %c, valid letters are \"%s\"", c, _alphabet);
+          "invalid letter %c, valid letters are \"%s\"", c, _alphabet.c_str());
     }
   }
 
