@@ -17,25 +17,30 @@
 
 // The purpose of this file is to test the CongruenceInterface class.
 
-// TODO IWYU!
+#include "catch.hpp"      // for REQUIRE
+#include "test-main.hpp"  // for LIBSEMIGROUPS_TEST_CASE
 
-#include "catch.hpp"  // for LIBSEMIGROUPS_TEST_CASE
-#include "libsemigroups/cong-pair.hpp"
-#include "libsemigroups/cong.hpp"
-#include "libsemigroups/element.hpp"
-#include "libsemigroups/fpsemi.hpp"
-#include "libsemigroups/knuth-bendix.hpp"
-#include "libsemigroups/tce.hpp"
-#include "libsemigroups/todd-coxeter.hpp"
-#include "test-main.hpp"
+#include "libsemigroups/cong-intf.hpp"     // for CongruenceInterface
+#include "libsemigroups/cong-pair.hpp"     // for CongruenceByPairs
+#include "libsemigroups/cong.hpp"          // for Congruence
+#include "libsemigroups/fpsemi.hpp"        // for FpSemigroup
+#include "libsemigroups/knuth-bendix.hpp"  // for KnuthBendix
+#include "libsemigroups/tce.hpp"           // for TCE
+#include "libsemigroups/todd-coxeter.hpp"  // for ToddCoxeter
+#include "libsemigroups/transf.hpp"        // for Transf<>
+
+CATCH_REGISTER_ENUM(libsemigroups::tril,
+                    libsemigroups::tril::TRUE,
+                    libsemigroups::tril::FALSE,
+                    libsemigroups::tril::unknown);
 
 namespace libsemigroups {
   struct LibsemigroupsException;  // Forward declaration
 
   constexpr bool REPORT              = false;
-  congruence_type constexpr twosided = congruence_type::twosided;
-  congruence_type constexpr left     = congruence_type::left;
-  congruence_type constexpr right    = congruence_type::right;
+  congruence_kind constexpr twosided = congruence_kind::twosided;
+  congruence_kind constexpr left     = congruence_kind::left;
+  congruence_kind constexpr right    = congruence_kind::right;
 
   using detail::TCE;
   using FroidurePinTCE = FroidurePin<TCE, FroidurePinTraits<TCE, TCE::Table>>;
@@ -50,18 +55,18 @@ namespace libsemigroups {
       std::unique_ptr<CongruenceInterface> cong;
 
       SECTION("ToddCoxeter") {
-        cong = detail::make_unique<ToddCoxeter>(twosided);
+        cong = std::make_unique<ToddCoxeter>(twosided);
       }
       SECTION("KnuthBendix") {
-        cong = detail::make_unique<KnuthBendix>();
+        cong = std::make_unique<KnuthBendix>();
       }
       SECTION("CongruenceByPairs") {
         ToddCoxeter tc(twosided);
-        tc.set_nr_generators(2);
+        tc.set_number_of_generators(2);
         tc.add_pair({0, 0, 0}, {0});
         tc.add_pair({1, 1, 1, 1}, {1});
         tc.add_pair({0, 1, 0, 1}, {0, 0});
-        cong = detail::make_unique<CongruenceByPairs<FroidurePinTCE>>(
+        cong = std::make_unique<CongruenceByPairs<FroidurePinTCE>>(
             right, tc.quotient_froidure_pin());
         // FIXME(when CongruenceByPairs has a proper traits class) We can't use
         // "twosided" here because TCE's products are only defined on the right
@@ -73,16 +78,16 @@ namespace libsemigroups {
       SECTION("Congruence") {
         FpSemigroup S;  // free semigroup
         S.set_alphabet(2);
-        cong = detail::make_unique<Congruence>(twosided, S);
+        cong = std::make_unique<Congruence>(twosided, S);
       }
 
-      cong->set_nr_generators(2);
+      cong->set_number_of_generators(2);
       cong->add_pair({0, 0, 0}, {0});
       cong->add_pair({1, 1, 1, 1}, {1});
       cong->add_pair({0, 1, 0, 1}, {0, 0});
 
       REQUIRE(!cong->finished());
-      REQUIRE(cong->nr_classes() == 27);
+      REQUIRE(cong->number_of_classes() == 27);
       REQUIRE(cong->finished());
       REQUIRE(cong->started());
       REQUIRE_THROWS_AS(cong->add_pair({0}, {1}), LibsemigroupsException);
@@ -97,10 +102,10 @@ namespace libsemigroups {
       std::unique_ptr<CongruenceInterface> cong;
       SECTION("no parent") {
         SECTION("ToddCoxeter") {
-          cong = detail::make_unique<ToddCoxeter>(twosided);
+          cong = std::make_unique<ToddCoxeter>(twosided);
         }
         SECTION("KnuthBendix") {
-          cong = detail::make_unique<KnuthBendix>();
+          cong = std::make_unique<KnuthBendix>();
         }
         REQUIRE_THROWS_AS(cong->const_contains({0}, {1}),
                           LibsemigroupsException);
@@ -111,15 +116,15 @@ namespace libsemigroups {
         SECTION("Congruence") {
           FpSemigroup S;  // free semigroup
           S.set_alphabet(2);
-          cong = detail::make_unique<Congruence>(twosided, S);
+          cong = std::make_unique<Congruence>(twosided, S);
         }
         SECTION("CongruenceByPairs") {
           ToddCoxeter tc(twosided);
-          tc.set_nr_generators(2);
+          tc.set_number_of_generators(2);
           tc.add_pair({0, 0, 0}, {0});
           tc.add_pair({1, 1, 1, 1}, {1});
           tc.add_pair({0, 1, 0, 1}, {0, 0});
-          cong = detail::make_unique<CongruenceByPairs<FroidurePinTCE>>(
+          cong = std::make_unique<CongruenceByPairs<FroidurePinTCE>>(
               right, tc.quotient_froidure_pin());
           // FIXME(when CongruenceByPairs has a proper traits class) We can't
           // use "twosided" here because TCE's products are only defined on the
@@ -135,7 +140,7 @@ namespace libsemigroups {
       REQUIRE(!cong->finished());
       REQUIRE(!cong->started());
 
-      cong->set_nr_generators(2);
+      cong->set_number_of_generators(2);
       cong->add_pair({0, 0, 0}, {0});
       cong->add_pair({1, 1, 1, 1}, {1});
       cong->add_pair({0, 1, 0, 1}, {0, 0});
@@ -154,22 +159,22 @@ namespace libsemigroups {
       std::unique_ptr<CongruenceInterface> cong;
 
       SECTION("ToddCoxeter") {
-        cong = detail::make_unique<ToddCoxeter>(twosided);
+        cong = std::make_unique<ToddCoxeter>(twosided);
         REQUIRE_THROWS_AS(cong->less({0}, {1}), LibsemigroupsException);
         REQUIRE(!cong->has_parent_froidure_pin());
       }
       SECTION("KnuthBendix") {
-        cong = detail::make_unique<KnuthBendix>();
+        cong = std::make_unique<KnuthBendix>();
         REQUIRE_THROWS_AS(cong->less({0}, {1}), LibsemigroupsException);
         REQUIRE(!cong->has_parent_froidure_pin());
       }
       SECTION("CongruenceByPairs") {
         ToddCoxeter tc(twosided);
-        tc.set_nr_generators(2);
+        tc.set_number_of_generators(2);
         tc.add_pair({0, 0, 0}, {0});
         tc.add_pair({1, 1, 1, 1}, {1});
         tc.add_pair({0, 1, 0, 1}, {0, 0});
-        cong = detail::make_unique<CongruenceByPairs<FroidurePinTCE>>(
+        cong = std::make_unique<CongruenceByPairs<FroidurePinTCE>>(
             right, tc.quotient_froidure_pin());
         // FIXME(when CongruenceByPairs has a proper traits class) We can't use
         // "twosided" here because TCE's products are only defined on the right
@@ -186,16 +191,16 @@ namespace libsemigroups {
       SECTION("Congruence") {
         FpSemigroup S;  // free semigroup
         S.set_alphabet(2);
-        cong = detail::make_unique<Congruence>(twosided, S);
+        cong = std::make_unique<Congruence>(twosided, S);
       }
 
-      cong->set_nr_generators(2);
+      cong->set_number_of_generators(2);
       cong->add_pair({0, 0, 0}, {0});
       cong->add_pair({1, 1, 1, 1}, {1});
       cong->add_pair({0, 1, 0, 1}, {0, 0});
 
       REQUIRE(cong->less({0}, {1}));
-      REQUIRE(cong->nr_classes() == 27);
+      REQUIRE(cong->number_of_classes() == 27);
     }
 
     LIBSEMIGROUPS_TEST_CASE("CongruenceInterface",
@@ -207,17 +212,17 @@ namespace libsemigroups {
       std::unique_ptr<CongruenceInterface> cong;
 
       SECTION("ToddCoxeter") {
-        cong = detail::make_unique<ToddCoxeter>(twosided);
+        cong = std::make_unique<ToddCoxeter>(twosided);
       }
       SECTION("KnuthBendix") {
-        cong = detail::make_unique<KnuthBendix>();
+        cong = std::make_unique<KnuthBendix>();
       }
 
       REQUIRE(!cong->is_quotient_obviously_infinite());
       // So far cong is not-defined, and hence not finite or infinite
       REQUIRE(!cong->is_quotient_obviously_finite());
 
-      cong->set_nr_generators(2);
+      cong->set_number_of_generators(2);
       REQUIRE(cong->is_quotient_obviously_infinite());
       REQUIRE(!cong->is_quotient_obviously_finite());
 
@@ -228,7 +233,7 @@ namespace libsemigroups {
       REQUIRE(!cong->is_quotient_obviously_infinite());
       REQUIRE(!cong->is_quotient_obviously_finite());
 
-      REQUIRE(cong->nr_classes() == 27);
+      REQUIRE(cong->number_of_classes() == 27);
       REQUIRE(!cong->is_quotient_obviously_infinite());
       REQUIRE(cong->is_quotient_obviously_finite());
     }
@@ -242,11 +247,11 @@ namespace libsemigroups {
       std::unique_ptr<CongruenceInterface> cong;
       SECTION("CongruenceByPairs") {
         ToddCoxeter tc(twosided);
-        tc.set_nr_generators(2);
+        tc.set_number_of_generators(2);
         tc.add_pair({0, 0, 0}, {0});
         tc.add_pair({1, 1, 1, 1}, {1});
         tc.add_pair({0, 1, 0, 1}, {0, 0});
-        cong = detail::make_unique<CongruenceByPairs<FroidurePinTCE>>(
+        cong = std::make_unique<CongruenceByPairs<FroidurePinTCE>>(
             right, tc.quotient_froidure_pin());
       }
       SECTION("Congruence") {
@@ -257,16 +262,17 @@ namespace libsemigroups {
         S.add_rule({0, 1, 0, 1}, {0, 0});
         REQUIRE(!S.is_obviously_infinite());
         REQUIRE(!S.is_obviously_finite());
-        cong = detail::make_unique<Congruence>(right, S);
+        cong = std::make_unique<Congruence>(right, S);
 
-        REQUIRE_THROWS_AS(cong->set_nr_generators(3), LibsemigroupsException);
+        REQUIRE_THROWS_AS(cong->set_number_of_generators(3),
+                          LibsemigroupsException);
       }
 
       cong->add_pair({0, 0, 0}, {0, 0});
 
       REQUIRE(!cong->is_quotient_obviously_infinite());
       REQUIRE(!cong->is_quotient_obviously_finite());
-      REQUIRE(cong->nr_classes() == 24);
+      REQUIRE(cong->number_of_classes() == 24);
       REQUIRE(!cong->is_quotient_obviously_infinite());
       REQUIRE(cong->is_quotient_obviously_finite());
     }
@@ -280,16 +286,16 @@ namespace libsemigroups {
       std::unique_ptr<CongruenceInterface> cong;
 
       SECTION("ToddCoxeter") {
-        cong = detail::make_unique<ToddCoxeter>(twosided);
+        cong = std::make_unique<ToddCoxeter>(twosided);
       }
       SECTION("KnuthBendix") {
-        cong = detail::make_unique<KnuthBendix>();
+        cong = std::make_unique<KnuthBendix>();
       }
       REQUIRE_THROWS_AS(cong->parent_froidure_pin(), LibsemigroupsException);
     }
 
     LIBSEMIGROUPS_TEST_CASE("CongruenceInterface",
-                            "009",
+                            "006",
                             "parent_froidure_pin",
                             "[quick]") {
       auto rg = ReportGuard(REPORT);
@@ -297,11 +303,11 @@ namespace libsemigroups {
       std::unique_ptr<CongruenceInterface> cong;
       SECTION("CongruenceByPairs") {
         ToddCoxeter tc(twosided);
-        tc.set_nr_generators(2);
+        tc.set_number_of_generators(2);
         tc.add_pair({0, 0, 0}, {0});
         tc.add_pair({1, 1, 1, 1}, {1});
         tc.add_pair({0, 1, 0, 1}, {0, 0});
-        cong = detail::make_unique<CongruenceByPairs<FroidurePinTCE>>(
+        cong = std::make_unique<CongruenceByPairs<FroidurePinTCE>>(
             left, tc.quotient_froidure_pin());
       }
 
@@ -313,8 +319,9 @@ namespace libsemigroups {
         S.add_rule({0, 1, 0, 1}, {0, 0});
         REQUIRE(!S.is_obviously_infinite());
         REQUIRE_NOTHROW(S.froidure_pin());
-        cong = detail::make_unique<Congruence>(left, S);
-        REQUIRE_THROWS_AS(cong->set_nr_generators(3), LibsemigroupsException);
+        cong = std::make_unique<Congruence>(left, S);
+        REQUIRE_THROWS_AS(cong->set_number_of_generators(3),
+                          LibsemigroupsException);
       }
 
       cong->add_pair({0, 0, 0}, {0, 0});
@@ -324,32 +331,30 @@ namespace libsemigroups {
     }
 
     LIBSEMIGROUPS_TEST_CASE("CongruenceInterface",
-                            "006",
+                            "007",
                             "cbegin/cend_ntc",
                             "[quick]") {
       auto rg = ReportGuard(REPORT);
-      auto S  = FroidurePin<Transformation<uint16_t>>(
-          {Transformation<uint16_t>({1, 3, 4, 2, 3}),
-           Transformation<uint16_t>({3, 2, 1, 3, 3})});
+      auto S  = FroidurePin<Transf<>>(
+          {Transf<>({1, 3, 4, 2, 3}), Transf<>({3, 2, 1, 3, 3})});
 
       std::unique_ptr<CongruenceInterface> cong;
 
       REQUIRE(S.size() == 88);
-      REQUIRE(S.nr_rules() == 18);
+      REQUIRE(S.number_of_rules() == 18);
 
       SECTION("ToddCoxeter") {
-        cong = detail::make_unique<ToddCoxeter>(twosided, S);
+        cong = std::make_unique<ToddCoxeter>(twosided, S);
       }
       SECTION("KnuthBendix") {
-        cong = detail::make_unique<KnuthBendix>(S);
+        cong = std::make_unique<KnuthBendix>(S);
       }
 
-      cong->add_pair(
-          S.factorisation(Transformation<uint16_t>({3, 4, 4, 4, 4})),
-          S.factorisation(Transformation<uint16_t>({3, 1, 3, 3, 3})));
+      cong->add_pair(S.factorisation(Transf<>({3, 4, 4, 4, 4})),
+                     S.factorisation(Transf<>({3, 1, 3, 3, 3})));
 
-      REQUIRE(cong->nr_classes() == 21);
-      REQUIRE(cong->nr_non_trivial_classes() == 1);
+      REQUIRE(cong->number_of_classes() == 21);
+      REQUIRE(cong->number_of_non_trivial_classes() == 1);
       REQUIRE(cong->cbegin_ntc()->size() == 68);
       std::vector<word_type> expect = {{0, 0, 1},
                                        {1, 0, 1},
@@ -425,33 +430,29 @@ namespace libsemigroups {
     }
 
     LIBSEMIGROUPS_TEST_CASE("CongruenceInterface",
-                            "007",
+                            "008",
                             "cbegin/cend_ntc",
                             "[quick]") {
       auto rg = ReportGuard(REPORT);
-      auto S  = FroidurePin<Transformation<uint16_t>>(
-          {Transformation<uint16_t>({1, 3, 4, 2, 3}),
-           Transformation<uint16_t>({3, 2, 1, 3, 3})});
+      auto S  = FroidurePin<Transf<>>(
+          {Transf<>({1, 3, 4, 2, 3}), Transf<>({3, 2, 1, 3, 3})});
 
       std::unique_ptr<CongruenceInterface> cong;
 
       REQUIRE(S.size() == 88);
-      REQUIRE(S.nr_rules() == 18);
+      REQUIRE(S.number_of_rules() == 18);
 
       SECTION("CongruenceByPairs") {
-        cong
-            = detail::make_unique<CongruenceByPairs<decltype(S)::element_type>>(
-                right, S);
+        cong = std::make_unique<CongruenceByPairs<decltype(S)>>(right, S);
       }
       SECTION("Congruence") {
-        cong = detail::make_unique<Congruence>(right, S);
+        cong = std::make_unique<Congruence>(right, S);
       }
-      cong->add_pair(
-          S.factorisation(Transformation<uint16_t>({3, 4, 4, 4, 4})),
-          S.factorisation(Transformation<uint16_t>({3, 1, 3, 3, 3})));
+      cong->add_pair(S.factorisation(Transf<>({3, 4, 4, 4, 4})),
+                     S.factorisation(Transf<>({3, 1, 3, 3, 3})));
 
-      REQUIRE(cong->nr_classes() == 72);
-      REQUIRE(cong->nr_non_trivial_classes() == 4);
+      REQUIRE(cong->number_of_classes() == 72);
+      REQUIRE(cong->number_of_non_trivial_classes() == 4);
       std::vector<size_t> actual(4, 0);
       std::transform(cong->cbegin_ntc(),
                      cong->cend_ntc(),
@@ -463,7 +464,7 @@ namespace libsemigroups {
     }
 
     LIBSEMIGROUPS_TEST_CASE("CongruenceInterface",
-                            "008",
+                            "009",
                             "quotient is immutable",
                             "[quick]") {
       auto rg = ReportGuard(REPORT);
@@ -471,22 +472,22 @@ namespace libsemigroups {
       std::unique_ptr<CongruenceInterface> cong;
 
       SECTION("ToddCoxeter") {
-        cong = detail::make_unique<ToddCoxeter>(twosided);
+        cong = std::make_unique<ToddCoxeter>(twosided);
       }
       SECTION("KnuthBendix") {
-        cong = detail::make_unique<KnuthBendix>();
+        cong = std::make_unique<KnuthBendix>();
       }
-      // TODO not yet implemented
+      // TODO(later) not yet implemented
       // SECTION("CongruenceByPairs") {
       //   cong =
-      //   detail::make_unique<CongruenceByPairs<decltype(S)::element_type>>(twosided,
+      //   std::make_unique<CongruenceByPairs<decltype(S)::element_type>>(twosided,
       //   S);
       // }
       SECTION("Congruence") {
-        cong = detail::make_unique<Congruence>(twosided);
+        cong = std::make_unique<Congruence>(twosided);
       }
 
-      cong->set_nr_generators(2);
+      cong->set_number_of_generators(2);
       cong->add_pair({0, 0, 0}, {0});
       cong->add_pair({1, 1, 1, 1}, {1});
       cong->add_pair({0, 1, 0, 1}, {0, 0});
@@ -500,33 +501,33 @@ namespace libsemigroups {
                             "[quick][cong]") {
       auto                                 rg = ReportGuard(REPORT);
       std::unique_ptr<CongruenceInterface> cong;
-      SECTION("KnuthBendix") {
-        cong = detail::make_unique<KnuthBendix>();
-        cong->set_nr_generators(4);
-        // KnuthBendix can find the class index, but the others can't
-        REQUIRE(cong->word_to_class_index({2, 2, 2, 2}) == 254);
-        REQUIRE(cong->class_index_to_word(2) == word_type({2}));
-      }
-
       SECTION("common behaviour") {
+        SECTION("KnuthBendix") {
+          cong = std::make_unique<KnuthBendix>();
+          cong->set_number_of_generators(4);
+          // KnuthBendix can find the class index, but the others can't
+        }
         SECTION("Congruence") {
-          cong = detail::make_unique<Congruence>(twosided);
-          cong->set_nr_generators(4);
+          cong = std::make_unique<Congruence>(twosided);
+          cong->set_number_of_generators(4);
+          static_cast<Congruence*>(cong.get())->max_threads(2);
           REQUIRE(cong->const_contains({1}, {2, 2, 2, 2, 2, 2, 2, 2, 2, 2})
                   == tril::FALSE);
         }
-        SECTION("ToddCoxeter") {
-          cong = detail::make_unique<ToddCoxeter>(twosided);
-          cong->set_nr_generators(4);
-          REQUIRE(cong->const_contains({1}, {2, 2, 2, 2, 2, 2, 2, 2, 2, 2})
-                  == tril::unknown);
-        }
+        REQUIRE(cong->word_to_class_index({2, 2, 2, 2}) == 254);
+        REQUIRE(cong->class_index_to_word(2) == word_type({2}));
+      }
+      SECTION("ToddCoxeter") {
+        cong = std::make_unique<ToddCoxeter>(twosided);
+        cong->set_number_of_generators(4);
+        REQUIRE(cong->const_contains({1}, {2, 2, 2, 2, 2, 2, 2, 2, 2, 2})
+                == tril::unknown);
         REQUIRE_THROWS_AS(cong->word_to_class_index({2, 2, 2, 2}),
                           LibsemigroupsException);
         REQUIRE_THROWS_AS(cong->class_index_to_word(2), LibsemigroupsException);
       }
       REQUIRE(!cong->contains({1}, {2, 2, 2, 2, 2, 2, 2, 2, 2, 2}));
-      REQUIRE(cong->nr_classes() == POSITIVE_INFINITY);
+      REQUIRE(cong->number_of_classes() == POSITIVE_INFINITY);
     }
 
     LIBSEMIGROUPS_TEST_CASE("CongruenceInterface",
@@ -536,21 +537,22 @@ namespace libsemigroups {
       auto                                 rg = ReportGuard(REPORT);
       std::unique_ptr<CongruenceInterface> cong;
       SECTION("ToddCoxeter") {
-        cong = detail::make_unique<ToddCoxeter>(twosided);
+        cong = std::make_unique<ToddCoxeter>(twosided);
       }
       SECTION("KnuthBendix") {
-        cong = detail::make_unique<KnuthBendix>();
+        cong = std::make_unique<KnuthBendix>();
       }
       SECTION("Congruence") {
-        cong = detail::make_unique<Congruence>(twosided);
+        cong = std::make_unique<Congruence>(twosided);
       }
-      REQUIRE_THROWS_AS(cong->set_nr_generators(0), LibsemigroupsException);
+      REQUIRE_THROWS_AS(cong->set_number_of_generators(0),
+                        LibsemigroupsException);
       REQUIRE_THROWS_AS(
           cong->const_contains({1}, {2, 2, 2, 2, 2, 2, 2, 2, 2, 2}),
           LibsemigroupsException);
       REQUIRE_THROWS_AS(cong->contains({1}, {2, 2, 2, 2, 2, 2, 2, 2, 2, 2}),
                         LibsemigroupsException);
-      REQUIRE(cong->nr_classes() == UNDEFINED);
+      REQUIRE(cong->number_of_classes() == UNDEFINED);
       REQUIRE_THROWS_AS(cong->word_to_class_index({2, 2, 2, 2}),
                         LibsemigroupsException);
       REQUIRE_THROWS_AS(cong->class_index_to_word(0), LibsemigroupsException);
@@ -564,22 +566,22 @@ namespace libsemigroups {
                             "no generating pairs",
                             "[quick][cong]") {
       auto rg = ReportGuard(REPORT);
-      auto S  = FroidurePin<Transformation<uint16_t>>(
-          {Transformation<uint16_t>({1, 3, 4, 2, 3}),
-           Transformation<uint16_t>({1, 3, 4, 2, 3}),
-           Transformation<uint16_t>({3, 2, 1, 3, 3})});
+      auto S  = FroidurePin<Transf<>>({Transf<>({1, 3, 4, 2, 3}),
+                                       Transf<>({1, 3, 4, 2, 3}),
+                                       Transf<>({3, 2, 1, 3, 3})});
 
       REQUIRE(S.size() == 88);
-      REQUIRE(S.nr_rules() == 21);
+      REQUIRE(S.number_of_rules() == 21);
 
-      CongruenceByPairs<decltype(S)::element_type> cong(twosided, S);
-      REQUIRE_THROWS_AS(cong.set_nr_generators(0), LibsemigroupsException);
+      CongruenceByPairs<decltype(S)> cong(twosided, S);
+      REQUIRE_THROWS_AS(cong.set_number_of_generators(0),
+                        LibsemigroupsException);
       REQUIRE(cong.word_to_class_index({2, 2, 2, 2}) == 0);
       REQUIRE(cong.const_contains({1}, {2, 2, 2, 2, 2, 2, 2, 2, 2, 2})
               == tril::FALSE);
-      REQUIRE_THROWS_AS(cong.class_index_to_word(2), LibsemigroupsException);
+      REQUIRE(cong.class_index_to_word(2) == word_type({2}));
       REQUIRE(!cong.contains({1}, {2, 2, 2, 2, 2, 2, 2, 2, 2, 2}));
-      REQUIRE(cong.nr_classes() == 88);
+      REQUIRE(cong.number_of_classes() == 88);
     }
   }  // namespace congruence
 }  // namespace libsemigroups
